from .base import process_response, ENDPOINT, any_required_kwargs


class ListMixin:
    @process_response
    def get_list(self, list_id, **kwargs):
        """
        GET /list/{list_id}
        """

        url = f"{ENDPOINT}/4/list/{list_id}"
        return self.make_request("GET", url, kwargs)

    @any_required_kwargs(["media_type", "media_id"])
    @process_response
    def get_list_item_status(self, list_id, **kwargs):
        """
        GET /list/{list_id}/item_status
        """

        url = f"{ENDPOINT}/4/list/{list_id}/item_status"
        return self.make_request("GET", url, kwargs)



from .base import ENDPOINT, process_response


class WatchProvidersMixin:
    @process_response
    def get_available_regions(self, **kwargs):
        """
        GET /watch/providers/regions
        """

        url = f"{ENDPOINT}/3/watch/providers/regions"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_movie_providers(self, **kwargs):
        """
        GET /watch/providers/movie
        """

        url = f"{ENDPOINT}/3/watch/providers/movie"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_tv_providers(self, **kwargs):
        """
        GET /watch/providers/tv
        """

        url = f"{ENDPOINT}/3/watch/providers/tv"
        return self.make_request("GET", url, kwargs)



from .base import process_response, ENDPOINT


class AuthMixin:
    @process_response
    def create_request_token(self, redirect_to="http://www.themoviedb.org/"):
        """
        POST /auth/request_token
        """

        url = f"{ENDPOINT}/4/auth/request_token"
        payload = {
            "redirect_to": redirect_to,
        }
        return self.make_request("POST", url, {}, payload)

    @process_response
    def create_access_token(self, request_token):
        """
        POST /auth/access_token
        """

        url = f"{ENDPOINT}/4/auth/access_token"
        payload = {
            "request_token": request_token,
        }
        return self.make_request("POST", url, {}, payload)



from .base import process_response, ENDPOINT


class AccountMixin:
    @process_response
    def get_account_lists(self, account_id, **kwargs):
        """
        GET /account/{account_id}/lists
        """

        url = f"{ENDPOINT}/4/account/{account_id}/lists"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_account_favorite_movies(self, account_id, **kwargs):
        """
        GET /account/{account_id}/movie/favorites
        """

        url = f"{ENDPOINT}/4/account/{account_id}/movie/favorites"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_account_favorite_tv_shows(self, account_id, **kwargs):
        """
        GET /account/{account_id}/tv/favorites
        """

        url = f"{ENDPOINT}/4/account/{account_id}/tv/favorites"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_account_movie_recommendations(self, account_id, **kwargs):
        """
        GET /account/{account_id}/movie/recommendations
        """

        url = f"{ENDPOINT}/4/account/{account_id}/movie/recommendations"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_account_tv_show_recommendations(self, account_id, **kwargs):
        """
        GET /account/{account_id}/tv/recommendations
        """

        url = f"{ENDPOINT}/4/account/{account_id}/tv/recommendations"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_account_movie_watchlist(self, account_id, **kwargs):
        """
        GET /account/{account_id}/movie/watchlist
        """

        url = f"{ENDPOINT}/4/account/{account_id}/movie/watchlist"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_account_tv_show_watchlist(self, account_id, **kwargs):
        """
        GET /account/{account_id}/tv/watchlist
        """

        url = f"{ENDPOINT}/4/account/{account_id}/tv/watchlist"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_account_rated_movies(self, account_id, **kwargs):
        """
        GET /account/{account_id}/movie/rated
        """

        url = f"{ENDPOINT}/4/account/{account_id}/movie/rated"
        return self.make_request("GET", url, kwargs)

    @process_response
    def get_account_rated_tv_shows(self, account_id, **kwargs):
        """
        GET /account/{account_id}/tv/rated
        """

        url = f"{ENDPOINT}/4/account/{account_id}/tv/rated"
        return self.make_request("GET", url, kwargs)



import requests

from .account import AccountMixin
from .auth import AuthMixin
from .list import ListMixin

mixins = [
    AccountMixin,
    AuthMixin,
    ListMixin,
]


class TMDb(*mixins):
    def __init__(self, access_token, **kwargs):
        self.access_token = access_token
        self.options = kwargs
        self.session = requests.Session()
        self.session.headers.update(
            {
                "Content-Type": "application/json;charset=utf-8",
                "Authorization": f"Bearer {access_token}",
            }
        )

    def make_request(self, method, url, request_params, payload=None):
        params = {}
        params.update(self.options)
        params.update(request_params)

        # https://developers.themoviedb.org/3/getting-started/image-languages
        # convert keyword argument: (..., include_image_language=["en", "null"])
        # into url param: ...&include_image_language=en,null
        for key in params:
            if type(params[key]) is list:
                params[key] = ",".join(params[key])

        return self.session.request(
            method, url, params=params, json=payload, timeout=30
        )

    def set_options(self, **kwargs):
        self.options.update(kwargs)

    def reset_options(self):
        self.options = {}

    def update_access_token(self, access_token):
        self.access_token = access_token



import os

from selenium.webdriver.common.by import By
from seleniumbase import BaseCase
from dotenv import load_dotenv
from tmdb.four import TMDb

load_dotenv()


class TMDb4TestClass(BaseCase):
    def test_v4_api(self):
        access_token = os.getenv("tmdb_access_token")
        username = os.getenv("tmdb_username")
        password = os.getenv("tmdb_password")

        msg = "Missing one or more configs (username, password, access_token)"
        if not all([username, password, access_token]):
            assert False, msg

        tmdb = TMDb(access_token)

        request_token = tmdb.create_request_token().get("request_token")

        self.open(
            f"https://www.themoviedb.org/auth/access?request_token={request_token}"
        )
        self.click_link("Login")
        self.type("username", username, By.ID)
        self.type("password", password, By.ID)
        self.click("login_button", by=By.ID)
        self.click('input[value="Approve"]')

        resp = tmdb.create_access_token(request_token)
        assert resp is not None

        user_access_token = resp.get("access_token")
        account_id = resp.get("account_id")

        tmdb.update_access_token(user_access_token)

        # /account
        assert tmdb.get_account_lists(account_id) is not None
        assert tmdb.get_account_favorite_movies(account_id) is not None
        assert tmdb.get_account_favorite_tv_shows(account_id) is not None

        # assert tmdb.get_account_movie_recommendations(account_id) is not None
        # assert tmdb.get_account_tv_show_recommendations(account_id) is not None

        assert tmdb.get_account_movie_watchlist(account_id) is not None
        assert tmdb.get_account_tv_show_watchlist(account_id) is not None
        assert tmdb.get_account_rated_movies(account_id) is not None
        assert tmdb.get_account_rated_tv_shows(account_id) is not None

        # /list
        assert tmdb.get_list(14105) is not None

        assert tmdb.get_list(14151) is not None

        assert (
            tmdb.get_list_item_status(
                14151,
                media_id=550,
                media_type="movie",
            )
            is not None
        )

        assert (
            tmdb.get_list_item_status(
                14105,
                media_id=1418,
                media_type="tv",
            )
            is not None
        )



from utils import polite


@polite
def test_get_available_regions(tmdb):
    assert tmdb.get_available_regions() is not None


@polite
def test_get_movie_providers(tmdb):
    assert tmdb.get_movie_providers() is not None


@polite
def test_get_tv_providers(tmdb):
    assert tmdb.get_tv_providers() is not None
